"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setActionTypes = exports.GitActions = exports.GitActionType = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const simple_git_1 = tslib_1.__importDefault(require("simple-git"));
const utils_1 = require("../../utils");
var GitActionType;
(function (GitActionType) {
    GitActionType["clone"] = "Clone git repository";
})(GitActionType = exports.GitActionType || (exports.GitActionType = {}));
var GitActions;
(function (GitActions) {
    GitActions.clone = (directory, seed) => tslib_1.__awaiter(this, void 0, void 0, function* () {
        if (!(0, utils_1.directoryExists)(directory)) {
            throw new Error(`Target directory does not exist: ${directory}`);
        }
        if ((0, utils_1.directoryExists)(path_1.default.join(directory, '.git'))) {
            throw new Error(`A git repository already exists in: ${directory}`);
        }
        if (!seed) {
            throw new Error(`Invalid seed information, unable to clone. Seed: ${seed}`);
        }
        /**
         * NOTE:
         * We may need to hide the spinner if user entry is required (ie. ssh key passphrase etc), but it's not clear
         * if we can even support this with simple-git. Seed repos will likely be public in future. Support for password
         * authentication was removed in Aug 2021. For now internal devs should create a new ssh key that is not passphrase
         * protected, add it to their github, and configure it for SSO against the @genesislcap github org. This git
         * actions / flow should not be used by external clients, as we can't onboard them into our github. As they will
         * only be onboarded into our JFrog, external clients should be using the download tar flow. See ./seed.ts actions.
         *
         * --------------------------------------------------------------------------------------------------------------
         *
         * Try as I might to make cloning work via JFrog to keep the setup the same for both external and internal users,
         * it seems it's just not possible until JFrog add the functionality to Artifactory, see:
         * https://stackoverflow.com/questions/66253739/clone-github-com-repo-through-artifactory-git-mirroring
         *
         * --------------------------------------------------------------------------------------------------------------
         *
         * Future ref notes:
         *
         * Should we want to try save credentials, review this:
         *
         * git config --global credential.helper cache
         * git config --global --unset credential.helper
         *
         * Here's an example of a fullyQualifiedRemoteRepo with token in place:
         *
         * `https://${username}:${token}@${repo}`;
         * `https://${token}@github.com/${scope}/${repo}`
         *
         * ...but again, this won't work with JFrog.
         *
         * Simple-git appears to use .netrc files which might be of some future use:
         *
         * machine github.com
         *     login <put-your-application-token-here>
         *     password x-oauth-basic
         *
         * Simple-git can take config options to update the header, ie: config: [`Authorization: token ${TOKEN}`]
         */
        const { remoteUser, remoteName, remoteRepo, remoteBranch, gitOptions } = seed;
        const fullyQualifiedRemoteRepo = `git@github.com:${remoteUser}/${remoteRepo}.git`;
        const options = {
            baseDir: directory,
            binary: 'git',
            maxConcurrentProcesses: 6,
        };
        const git = (0, simple_git_1.default)(options);
        yield git.init();
        yield git.addRemote(remoteName, fullyQualifiedRemoteRepo);
        yield git.pull(remoteName, remoteBranch, gitOptions);
    });
})(GitActions = exports.GitActions || (exports.GitActions = {}));
const setActionTypes = (plop) => {
    plop.setActionType(GitActionType.clone, (answers, config) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const { directory, seed } = config;
        yield GitActions.clone(directory, seed);
        return ''; // < There is a typing issue with Plop, these need to return a string
    }));
};
exports.setActionTypes = setActionTypes;
